
#property copyright "Copyright  2008, 2hrfx.com"
#property link      "http://www.2hrfx.com"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 LimeGreen
#property indicator_color2 Orange
#property indicator_color3 Orange
#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2

extern ENUM_TIMEFRAMES TimeFrame = PERIOD_CURRENT;
extern ENUM_APPLIED_PRICE Price = 0;
extern int Length = 20;
extern int Displace = 0;
extern int Filter = 0;
extern double Deviation = 0.0;
extern bool   alertsOn        = false;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;
extern bool   alertsNotify    = false;
double MABuffer[];
double DnBuffera[];
double DnBufferb[];
double trend[];
double alfa[];
int Phase;
int Len;
int Cycle = 4;
double Coeff;
double beta;
double t;
double sum;
double Weight;
double g;
double pi = 3.1415926535;
string indicatorFileName;
bool   returnBars;

int init() {
   IndicatorBuffers(4);
   SetIndexBuffer(0, MABuffer);
   SetIndexBuffer(1, DnBuffera);
   SetIndexBuffer(2, DnBufferb);
   SetIndexBuffer(3, trend);
   IndicatorDigits(MarketInfo(Symbol(), MODE_DIGITS));
   string short_name = "TCCI(" + Length + ")";
   IndicatorShortName(short_name);
   SetIndexShift(0, Displace);
   SetIndexShift(1, Displace);
   SetIndexShift(2, Displace);
   SetIndexDrawBegin(0, Length * Cycle + Length);
   SetIndexDrawBegin(1, Length * Cycle + Length);
   SetIndexDrawBegin(2, Length * Cycle + Length);
   Coeff = 3.0 * pi;
   Phase = Length - 1;
   Len = Length * Cycle + Phase;
   ArrayResize(alfa, Len);
   Weight = 0;
   for (int i = 0; i < Len - 1; i++) {
      if (i <= Phase - 1) t = 1.0 * i / (Phase - 1);
      else t = (i - Phase + 1) * (2.0 * Cycle - 1.0) / (Cycle * Length - 1.0) + 1.0;
      beta = MathCos(pi * t);
      g = 1.0 / (Coeff * t + 1.0);
      if (t <= 0.5) g = 1;
      alfa[i] = g * beta;
      Weight += alfa[i];
   }
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame == -99;
      TimeFrame         = MathMax(TimeFrame,_Period);
   return (0);
}

int start() {
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            int limit = MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { MABuffer[0] = limit+1; return(0); }
           if (TimeFrame!=Period())
           {
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
               for(int i=limit; i>=0; i--)
               {
                  int y = iBarShift(NULL,TimeFrame,Time[i]);               
                     MABuffer[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Price,Length,Displace,Filter,Deviation,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,0,y);
                     trend[i]    = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Price,Length,Displace,Filter,Deviation,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,3,y);
                     DnBuffera[i] = EMPTY_VALUE;
                     DnBufferb[i] = EMPTY_VALUE;
                     if (trend[i] == -1) PlotPoint(i,DnBuffera,DnBufferb,MABuffer);
               }
               return(0);
           }
    
   
   for (int shift = limit; shift >= 0; shift--) {
      sum = 0;
      for (i = 0; i <= Len - 1; i++) {
         double price = iMA(NULL,0,1,0,MODE_SMA,Price,shift+i);
         sum += alfa[i] * price;
      }
      if (Weight > 0.0) MABuffer[shift] = (Deviation / 100.0 + 1.0) * sum / Weight;
      if (Filter > 0)
         if (MathAbs(MABuffer[shift] - (MABuffer[shift + 1])) < Filter * Point) MABuffer[shift] = MABuffer[shift + 1];
         DnBuffera[shift] = EMPTY_VALUE;
         DnBufferb[shift] = EMPTY_VALUE;
         trend[shift] = trend[shift + 1];
         if (MABuffer[shift] - MABuffer[shift + 1] > Filter * Point) trend[shift] =  1;
         if (MABuffer[shift + 1] - MABuffer[shift] > Filter * Point) trend[shift] = -1;
         if (trend[shift] == -1) PlotPoint(shift,DnBuffera,DnBufferb,MABuffer);
   }
   manageAlerts();
   return (0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

        message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," TCCI slope changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)  SendMail(StringConcatenate(Symbol(),"TCCI"),message);
          if (alertsNotify)  SendNotification(message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}
